#!/bin/python3
#%%
import datetime

import pandas as pd
from sklearn.model_selection import train_test_split

from tools.data_processing.easy_pandas import t_print

def simple_split_idx(df: pd.DataFrame, y_col = 'y', test_size = .2):
    print(f'test_size = {test_size}')
    y_train, y_validate = train_test_split(
        df, df[y_col],
        test_size=test_size, 
        stratify=df[y_col], 
        random_state=0
        )[2:]
    train_idx = y_train.sort_index().index.to_list()
    validate_idx = y_validate.sort_index().index.to_list()
    return train_idx, validate_idx


def append_data_split_col(data_df: pd.DataFrame, test_date: str, y_col = 'y', test_size = .2, date_col = 'date', data_split_col = 'data_split'):
    if isinstance(test_date, str):
        test_date = datetime.datetime.strptime(test_date, '%Y-%m-%d') 
    data_df = data_df.loc[~data_df[y_col].isna(), :].copy()
    data_df[data_split_col] = pd.Series(dtype=str)
    test_idx = data_df[date_col] >= test_date
    data_df.loc[test_idx, data_split_col] = 'test'
    train_validate_df = data_df.loc[data_df[data_split_col].isna(), :]
    if test_size == 'auto':
        test_size = test_idx.sum() / (~test_idx).sum()
        t_print(f'test_size = {test_size}')
    train_idx, validate_idx = simple_split_idx(train_validate_df, y_col=y_col, test_size=test_size)
    data_df.loc[train_idx, data_split_col] = 'train'
    data_df.loc[validate_idx, data_split_col] = 'validate'

    gp1 = data_df.groupby(by=[data_split_col])
    gp2 = data_df.groupby(by=[data_split_col, y_col])
    stat_df = pd.DataFrame({
        'size': gp1[y_col].size(),
        'count': gp1[y_col].count(),
        'pos_ratio': gp1[y_col].mean(),
        'min_date': gp1[date_col].min(),
        'max_date': gp1[date_col].max(),
        'pct_by_split': gp1[y_col].count() / len(data_df),
    })
    print(stat_df)
    stat_df = pd.DataFrame({
        'size': gp2[y_col].size(),
        'count': gp2[y_col].count(),
        'rate': gp2[y_col].count() / gp1[y_col].size(),
        'pct_by_split': gp2[y_col].size() / gp1[y_col].size(),
        'min_date': gp2[date_col].min(),
        'max_date': gp2[date_col].max(),
    })
    print(stat_df)
    return data_df


def get_data_split_col(data_df: pd.DataFrame, test_date: str, y_col = 'y', test_size = .2, date_col = 'date', data_split_col = 'data_split'):
    data_split_col_sr = append_data_split_col(data_df, test_date, y_col, test_size, date_col, data_split_col = data_split_col)[data_split_col]
    return data_split_col_sr


#%%


def stat_y_by_time(ya_df: pd.DataFrame, by_table = [], time_col = 'month', y_col = 'y', prefix = None):
    if len(by_table) == 0:
        gp2 = ya_df.groupby(by=by_table + [time_col])
        df = pd.DataFrame({
            'size': gp2[y_col].size(),
            'count': gp2[y_col].count(),
            'coverage': gp2[y_col].count() / gp2[y_col].size(),
            'pos_cnt': gp2[y_col].sum(),
            'pos_ratio': gp2[y_col].mean(),
            'pct_by_group': gp2[y_col].size() / len(ya_df)
        })
    else:
        gp1 = ya_df.groupby(by=by_table)
        gp2 = ya_df.groupby(by=by_table + [time_col])
        df = pd.DataFrame({
            'size': gp2[y_col].size(),
            'count': gp2[y_col].count(),
            'coverage': gp2[y_col].count() / gp2[y_col].size(),
            'pos_cnt': gp2[y_col].sum(),
            'pos_ratio': gp2[y_col].mean(),
            'pct_by_group': gp2[y_col].size() / gp1[y_col].size()
        })
    df.sort_index(inplace=True)
    if prefix is not None:
        df = df.add_prefix(prefix)
    return df


def stat_y_by_data_split(ya_df: pd.DataFrame, by_table = [], data_split_col = 'data_split', y_col = 'y', date_col = 'date'):
    ya_df = ya_df.copy()
    if len(by_table) == 0:
        ya_df.insert(0, 'table', 'no_table')
        by_table = ['table']
    ya_df[data_split_col].fillna('NaN', inplace=True)
    gp1 = ya_df.groupby(by=by_table)
    gp2 = ya_df.groupby(by=by_table + [data_split_col])
    df = pd.DataFrame({
        'size': gp2[y_col].size(),
        'count': gp2[y_col].count(),
        'coverage': gp2[y_col].count() / gp2[y_col].size(),
        'pos_cnt': gp2[y_col].sum(),
        'pos_ratio': gp2[y_col].mean(),
        'pct_by_split': gp2[y_col].size() / gp1[y_col].size(),
        'min_date': gp2[date_col].min(),
        'max_date': gp2[date_col].max(),
    })
    df.sort_index(inplace=True)
    return df

